@@ -45,6 +45,7 @@ static char copyr[] = "Copyright (c) 1996,1997,1999,2001,2002,2003,2004,2009,201
 #include <sys/un.h>
 #include <net/if.h>
 #include <netinet/if_ether.h>
+#include <arpa/inet.h>
 #include <netinet/in.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
@@ -55,7 +56,8 @@ static char copyr[] = "Copyright (c) 1996,1997,1999,2001,2002,2003,2004,2009,201
 #include <time.h>
 #include <signal.h>
 #include <errno.h>
-#include <linux/ax25.h>
+#include <netax25/ax25.h>
+#include <netax25/axlib.h>
 #ifndef AX25_MTU
 #define AX25_MTU 256
 #endif
@@ -185,10 +187,12 @@ struct stuff {			/* maybe I should learn C++... */
 /* General options: */
 static int Verbose = 0;
 static int Testing = 0;
+static int Maxhops = 0;	/* limit direct input packets to this many digipeats */
 static int Digi_SSID = 0;
 static int Kill_dupes = 0;	/* kill dupes even in conventional mode */
 static int Kill_loops = 0;	/* kill loops */
 static int Doing_dupes = 0;	/* dupelist was set on some interface */
+static int Max_wide_n = 0;	/* The limit to wideN-N SSID value */
 static char *Logfile = NULL;
 static ax25_address Aprscall;	/* replace mic-e encoded to-call with this */
 static ax25_address Trace_dummy; /* dummy call for tracen-n substitution */
@@ -205,6 +209,7 @@ static char *Tag = NULL;	/* tag onto end of rx'd posit */
 static int Taglen = 0;
 static int Idinterval = (9*60)+30; /* default to 9:30 */
 static int Keep = 28;		/* seconds to remember for dupe test */
+
 static int I_flags = 0;		/* interface default flags */
 #define I_NEED_ID    0x01	/* need to ID */
 #define SUBST_MYCALL 0x02	/* replace digi alias w/mycall */
@@ -235,8 +240,7 @@ static int I_flags = 0;		/* interface default flags */
  *	RELAY*,TRACE3-3
  *	RELAY,N2YGK-7*,TRACE3-2
  *	RELAY,N2YGK-7,WB2ZII*,TRACE3-1
- *	RELAY,N2YGK-7,WB2ZII,N2MH-15*,TRACE3
- *	RELAY,N2YGK-7,WB2ZII,N2MH-15,WA2YSM-14*
+ *	RELAY,N2YGK-7,WB2ZII,N2MH-15*,TRACE3*
  *     (What happens when the digi list gets too long for the AX.25 protocol
  *     appears not to have been considered in this design.  I guess we'll
  *     just insert as many as we can.)
@@ -385,6 +389,7 @@ static int rx_to_me(struct stuff *s);
 static int rx_bud_deny(struct stuff *s);
 static int rx_from_me(struct stuff *s);
 static int rx_ssid(struct stuff *s);
+static int rx_limit(struct stuff *s);
 static int rx_flood(struct stuff *s);
 static int rx_digi(struct stuff *s);
 
@@ -435,6 +440,8 @@ rx_packet(struct interface *i,	/* which interface received on */
     return;
   else if (rx_to_me(&s))	/* Addressed to me? */
     return;
+  else if (rx_limit(&s))	/* Will this packet exceed the digipeat limit */
+    return;
   else if (rx_flood(&s))	/* flood special handling */
     return;
   else if (rx_digi(&s))		/* conventional digi */
@@ -608,6 +615,28 @@ intf_of(ax25_address *callsign)
   return NULL;
 }
 
+/* Does a packet appear to come directly to us from the rf originator */
+static int
+is_direct(struct stuff *s)
+{
+  struct callsign_list *c;
+  int flag=0, digit, n1, n2;
+  if (s->in.ax_next_digi != 0) return 0;
+  c = calltab_entry(&s->in.ax_digi_call[0],&flag);
+  if (!c) 	/* Nothing we recognize or will deal with, but prob. direct */
+     return 1;
+  if ((!flag&C_IS_FLOOD) || (!flag&C_IS_FLOODN)) /* alias, yes we are direct */
+     return 1;
+  digit = ((s->in.ax_digi_call[0].ax25_call[c->floodlen]) & 0xFE)>>1;
+  if ((digit<'0') || digit > '9') return 0; /* Something is very wrong */
+  n1 = digit - 0x30;
+  n2 = (s->in.ax_digi_call[0].ax25_call[ALEN]&SSID)>>1;
+  if (n2<n1) 	/* There's probably been a hop already */
+     return 0;
+
+  return 1;
+}
+
 static int
 rx_to_me(struct stuff *s)
 {
@@ -656,6 +685,60 @@ rx_from_me(struct stuff *s)
   return result;
 }  
 
+/* Does a direct inbound packet request more hops than the limit we set */
+static int
+rx_limit(struct stuff *s)
+{
+  int sum = 0, result = 0, flag=0;
+  struct callsign_list *c;
+  if (!Maxhops) return result;
+  if (!(is_direct(s))) return result;
+  c = calltab_entry(&s->in.ax_digi_call[0],&flag);
+  if (!c) return result;
+  if (Verbose) {
+    fprintf(stderr, "Direct input packet with known via call\n");
+  }
+
+  for (int n=0 ; n < s->in.ax_n_digis; n++) {
+    flag = 0;
+    c = calltab_entry(&s->in.ax_digi_call[n],&flag);
+    if (!c) {
+        sum++;
+	if (Verbose) {
+	   fprintf(stderr, "Call, non flood running total %i\n", ax25_ntoa_pretty(&s->in.ax_digi_call[n]), sum);
+	}
+        continue;
+    }
+    if ((flag&C_IS_FLOOD) && (flag&C_IS_FLOODN)) {
+        sum += (s->in.ax_digi_call[n].ax25_call[ALEN]&SSID)>>1;
+	if (Verbose) {
+	   fprintf(stderr, "Call %s, flag %i, flood running total %i\n", ax25_ntoa_pretty(&s->in.ax_digi_call[n]), flag, sum);
+	}
+    } else {
+	sum++;
+	if (Verbose) {
+	   fprintf(stderr, "Call %s, flag %i, non floodn running total %i\n", ax25_ntoa_pretty(&s->in.ax_digi_call[n]), flag, sum);
+	}
+    }
+  }
+
+  if (Verbose) {
+    fprintf(stderr,"Total hops expected for pkt being received directly %i limit %i\n",
+       sum, Maxhops);
+  }
+  if (sum > Maxhops) {
+     ++s->i->stats.rx_ign;
+     result = 1;
+  }
+
+  if (Verbose) {
+    fprintf(stderr,"Direct input packet%sdropped for too many digipeats requested\n",
+	    result?" ":" not ");
+  }
+
+  return result;
+}
+
 /* 
  * SSID path selection only applies if:
  *  1. SSID digipeating is enabled.
@@ -756,6 +839,10 @@ rx_flood(struct stuff *s)
       fprintf(stderr,"Got a flooding %s route.\n",
 	      ax25_ntoa_pretty(&s->in.ax_digi_call[s->in.ax_next_digi]));
     }
+    if ((Max_wide_n) && (wide > Max_wide_n)) {
+       wide = Max_wide_n;
+       if (Verbose) fprintf(stderr, "Flooding hopcount limited to %d\n", wide);
+    }
 
     /* flooding algorithm always kills dupes.  If kill_dupes
        option was not selected, then do the dupe checking here */
@@ -769,6 +856,9 @@ rx_flood(struct stuff *s)
     s->out = s->in;		/* copy the input header */
     s->out.ax_digi_call[s->out.ax_next_digi].ax25_call[ALEN] &= ~SSID;
     s->out.ax_digi_call[s->out.ax_next_digi].ax25_call[ALEN] |= ((--wide) << 1)&SSID;
+    /* Handle FLOOD-1 -> FLOOD-0 case */
+    if (wide == 0)
+      s->out.ax_digi_call[s->out.ax_next_digi].ax25_call[ALEN] |= REPEATED;
     /* TRACEn-n: insert dummy mycall in front: xmit will put the real one in */
     if ((thisflags&(C_IS_FLOODN|C_IS_TRACE)) == (C_IS_FLOODN|C_IS_TRACE)) {
       int n;
@@ -793,7 +883,7 @@ rx_flood(struct stuff *s)
     ++s->i->stats.flood;
     if (xmit(s,NULL) < 0) 
       perror("xmit");
-    return 1;
+    result = 1;
   }
   if (Verbose) {
     fprintf(stderr,"Did %s require special flooding handling.\n",
@@ -815,7 +905,7 @@ rx_digi(struct stuff *s)
     if (Verbose) {
       fprintf(stderr,"Got a conventional digipeat.\n");
     }
-      s->out = s->in;	/* copy input list to output unmodifed */
+      s->out = s->in;	/* copy input list to output unmodified */
       s->out.ax_digi_call[s->out.ax_next_digi].ax25_call[ALEN] |= REPEATED;
       ++s->i->stats.digi;
       if (xmit(s,NULL) < 0) 
@@ -1050,7 +1140,10 @@ xmit(struct stuff *s, struct interface_list *dupelist)
 	  fclose(of);
       }
       if (Verbose) {
-	fprintf(stderr,"%s: TX: ",l->i->port);
+	char buf[20];
+	time_t tick = time(0);
+	strftime(buf,sizeof(buf),"%T",gmtime(&tick));
+	fprintf(stderr,"%s %s: TX: ",buf,l->i->port);
 	print_it(stderr,&calls,op-(vecl[n]+l->i->taglen),vecl[n]+l->i->taglen);
       }
       ++l->i->stats.tx;
@@ -1076,7 +1169,7 @@ xmit(struct stuff *s, struct interface_list *dupelist)
  */
 struct pkt {
   struct pkt *next,*prev;
-  time_t t;			/* when recevied */
+  time_t t;			/* when received */
   ax25_address to;		/* destination */
   ax25_address fr;		/* source */
   int l;			/* length of text */
@@ -1214,7 +1307,7 @@ loop_packet(struct ax_calls *calls,u_char *cp,int len)
 
 static void
 cleanup(sig)
-int sig;			/* dont't really care what it was */
+int sig;			/* don't really care what it was */
 {
   if (alarm(0) > 0)		/* an alarm was pending, so... */
     sked_id(NULL);		/*  ID one last time */
@@ -1387,6 +1480,7 @@ enum {NONE=no_argument,REQD=required_argument,OPT=optional_argument};
 static struct option opts[] = {
   {"verbose",OPT,0,'v'},
   {"testing",NONE,0,'T'},
+  {"floodnlimit",REQD,0,'W'},
   {"interface",REQD,0,'p'},
   {"port",REQD,0,'p'},
   {"trace",REQD,0,'F'},
@@ -1398,6 +1492,7 @@ static struct option opts[] = {
   {"digipath",REQD,0,'d'},
   {"tag",REQD,0,'t'},
   {"keep",REQD,0,'k'},
+  {"hops",REQD,0,'H'},
   {"logfile",REQD,0,'l'},
   {"idinterval",REQD,0,'i'},
   {"subst_mycall",NONE,0,'C'},
@@ -1426,7 +1521,7 @@ static struct option opts[] = {
   {"duplicate",REQD,0,O('d')},
   {0,0,0,0}
 };
-static char *optstring = "CcMmXxF:f:n:s:e:w:t:k:l:i:d:p:DLVvT30o:B:b:";
+static char *optstring = "CcMmXxF:f:n:s:e:w:t:k:H:l:i:d:p:W:DLVvT30o:B:b:";
 
 static void
 do_opts(int argc, char **argv)
@@ -1446,6 +1541,10 @@ do_opts(int argc, char **argv)
     case 'T':
       ++Testing;
       break;
+    case 'W':
+      Max_wide_n = atoi(optarg);
+      if (Max_wide_n < 0) Max_wide_n = 0;
+      break;
     case 'p':			/* -p port[:alias,alias,alias...] */
       if (N_intf >= MAXINTF) {
 	fprintf(stderr,"too many interfaces!\n");
@@ -1566,6 +1665,10 @@ do_opts(int argc, char **argv)
       if ((Keep = atoi(optarg)) <= 0)
 	Keep = 28;		/* default keep is 28 */
       break;
+    case 'H':
+      Maxhops = atoi(optarg);
+      if (Maxhops < 0) Maxhops = 0;
+      break;
     case 'l':
       Logfile = optarg;		/* log digipeated packets */
       break;
@@ -1681,6 +1784,7 @@ usage()
   fprintf(stderr," general options:\n");
   fprintf(stderr," -v | --verbose     -- produce verbose debugging output\n");
   fprintf(stderr," -T | --testing     -- test mode: listen to my packets too\n");
+  fprintf(stderr," -W | --floodnlimit -- limit to the maximum hops n in FLOODN-n\n");
   fprintf(stderr," -D | --kill_dupes  -- suppress Duplicate packets\n");
   fprintf(stderr," -L | --kill_loops  -- suppress Looping packets\n");
   fprintf(stderr," -V | --version     -- print program version and exit\n");
@@ -1689,6 +1793,7 @@ usage()
   fprintf(stderr," -f | --flood       -- set Flooding callsign (usually WIDE)\n");
   fprintf(stderr," -F | --trace       -- set Flooding TRACE callsign (usually TRACE)\n");
   fprintf(stderr," -k | --keep        -- seconds of old dupes to remember\n");
+  fprintf(stderr," -H | --hops        -- direct input maximum number of future hops allowed\n");
   fprintf(stderr," -l | --logfile     -- log digipeated packets here\n");
   fprintf(stderr," per-interface options (put *before* each -p as needed):\n");
   fprintf(stderr," -C (-c) | --[no]subst_mycall  -- do (not) perform Mycall substitution\n");
@@ -2133,12 +2238,17 @@ check_config()
   }
   /* flags */
   printf("keep dupes for: %d seconds\n",Keep);
+  if (Maxhops) {
+     printf("maximum number of hops allowed is %d for packets input directly to us\n",Maxhops);
+  }
   printf("log file: %s\n",(Logfile)?Logfile:"(none)");
 #define onoff(x) (x)?"ON":"OFF"
   printf("kill dupes: %s loops: %s  testing: %s\n",
 	 onoff(Kill_dupes), onoff(Kill_loops), 
 	 onoff(Testing));
 #undef onoff
+  if (Max_wide_n)
+     printf("Maximum value for n in FloodN-n %d\n", Max_wide_n);
   fflush(stdout);
 }
 
