@@ -61,9 +61,8 @@
 #include <time.h>
 #include <signal.h>
 
-#include <netax25/ax25.h>
-#include <netax25/axlib.h>
-#include <netax25/axconfig.h>
+#include <linux/ax25.h>
+#include "netax25/axconfig.h"
 
 #ifndef HAVE_LIBAX25_EXTENSIONS
 #include "libax25ext.h"
@@ -97,7 +96,7 @@ char *infofile = "/var/ax25/aprsmon.info";
 #endif
 
 #ifndef VERSION
-#define VERSION "$Revision: 1.2 $"
+#define VERSION "$Revision$"
 #endif
 
 char *title = "Live data from Linux";
@@ -188,6 +187,7 @@ int main(int argc, char **argv)
     uname(&me);
     printf("aprsmon>JAVA:javaMSG  :Linux APRS server (%s-%s by Alan Crosswell, N2YGK) on %s running %s %s\r\n",
 	   PACKAGE, VERSION, me.nodename, me.sysname, me.release);
+    printf("# %s\r\n",title);
 #ifdef USE_SHM
     (void) shm_slave(stdout,infofile);
 #endif
